/* Copyright (C) 2007-2009 Loongtek.com
 * All rights reserved.
 *
 * This library is a P2PTunnel toolkit, which features as below:
 *
 * 1.Support ICE, STUN and Relay protocols for traversing NATs.
 * 2.Embedded STUN and Relay server
 * 3.Select a best connection from multiple available paths between two endpoints.
 * 4.Provide reliable packets delivery mechanism based on ICE connections
 * 5.Encrypt/Decrypt packets using Blowfish algorithm.
 *
 * Revision
 * --------------------------------------------------------------
 * Date         |    Author     |            Note
 * --------------------------------------------------------------
 * 2007-09-18   |    George     |      Init this library 
 * --------------------------------------------------------------
 */

#ifndef __TRANSPORT_H__
#define __TRANSPORT_H__

#define EXPORTDLL __declspec( dllexport )
#define MAX_HOST_NAME_LENGTH 64

/******************************************
 * Return values of functions
 ******************************************/
#define T_SUCCESS 0
#define T_ERROR   -1

/******************************************
 * Transport type of sockets
 ******************************************/
#define TR_SOCK_STREAM 1              /*< TCP */
#define TR_SOCK_DGRAM  2              /*< UDP */

typedef int TrSessionID;              /*< ICE session identify */

/******************************************
 * Represent a endpoint address
 ******************************************/
typedef struct TrSocketAddress_ {
	char host[MAX_HOST_NAME_LENGTH];  /*< Endpoint IP address or FQDN */
	unsigned short  port;             /*< Endpoint Port */
} TrSocketAddress;

/******************************************
 * Represent a candidate defined ICE 
 ******************************************/
typedef struct TrCandidate_ {
  char *name;                        /*< Candidate name */
  char *protocol;                    /*< Protocol string such as udp, tcp, stun, relay and tls */
  TrSocketAddress address;           /*< Candidate's IP/Port pair */
  float preference;                  /*< Candidate's preference used in best connection algorithm */
  char *username;                    /*< Candidate's temporarily user name */
  char *password;                    /*< Candidate's temporarily password */
  char *type;                        /*< Candidate's type if have */
  char *network_name;                /*< Candidate's network name if have */
  unsigned int generation;           /*< When receiving a new cadidate which generation is greater than 
									     cadidates' in current list, the old candidates will be droped */
} TrCandidate;

/******************************************
 * Candidates Array 
 ******************************************/
typedef struct TrCandidateArray_ {
	int count;                       /*< Candidate size in the array*/
	TrCandidate *candidates;         /*< Pointer to the first candidate, All candidates put into 
									     a continuous memory space */
} TrCandidateArray;

/******************************************
 * ICE session state
 ******************************************/
typedef enum TrSessionState_ {
	TR_STATE_SESSION_CREATED,        /*< ICE session created */
	TR_STATE_SESSION_DESTROYED,      /*< ICE session destroyed */
	TR_STATE_CONNECTING,             /*< ICE session connecting to remote endpoint */
    TR_STATE_CONNECTED,              /*< ICE session connected */
	TR_STATE_NETWORK_ERROR           /*< Network error or too many packets lost under reliable session mode */
} TrSessionState;

/******************************************
 * Helper class only for SDK internal used
 ******************************************/
class TrICESocketHelper;

/******************************************
 * TrICESocket represent a ICE connection endpoint
 ******************************************/
class EXPORTDLL TrICESocket {
public:
	/******************************************************
	 * Parameters:
	 * reliable -- if true the TrICESocket will support 
	 *             restransmiting and reordering mechanism
	 * key      -- if keylen > 0 then this point to the 
	 *             security key
	 * keylen   -- key length, if greater than 0 means the 
	 *             TrICESocket should support encrypt/decrypt
	 *             packets
	 ******************************************************/
	TrICESocket(bool reliable, unsigned char *key, int keylen);
	~TrICESocket();

	/******************************************************
	 * Parameters:
	 * data     -- point to packet buffer header
	 * len      -- received packet length
	 ******************************************************/
	virtual void OnReceivePacketData(const char *data, int len) = 0;   

	/******************************************************
	 * Parameters:
	 * state    -- TrICESocket state event
	 ******************************************************/              
	virtual void OnSessionState(TrSessionState state) = 0;    

	/******************************************************
	 * When one or more ports are ready for receive data the 
	 * TrICESocket send this msg to App 
	 * Parameters:
	 * candidates    -- point to the header of the candidate array
	 * count         -- candidate array size
	 *******************************************************/ 
	virtual void OnCandidatesReady(TrCandidate *candidates, int count) = 0;   

	/*******************************************************
	 * App send packets to SDK 
	 * Parameters:
	 * data      -- packet buffer
	 * data_len  -- packet length
	 *******************************************************/ 
	void SendPacketData(const char *data, int data_len);   

	/*******************************************************
	 * When App receive remote endpoint address  then call 
	 * this interface to notify TrICESocket
	 * Parameters:
	 * candidates   -- point to the header of the candidate array
	 * count        -- candidate array size
	 *******************************************************/ 
	void AddRemoteCandidates(TrCandidate *candidates, int count); 

	/*******************************************************
	 * Get this TrICESocket identify it is unique
	 * in the SDK
	 *******************************************************/
	TrSessionID GetSessionID();
	TrICESocketHelper *GetHelper();

public: 
	friend class TrICESocketHelper;

private:
	TrSessionID       sid_;           /*< TrICESocket(ICE Session) identify */
	TrICESocketHelper *helper_;       /*< for SDK */
	bool reliable_enable_;            /*< for SDK */
	unsigned char *key_;              /*< for SDK */
	int keylen_;                      /*< for SDK */
};

/******************************************
 * Helper class only for SDK internal used
 ******************************************/
class TrAsyncSocketHelper;

/******************************************
 * Asynchronous socket
 ******************************************/
class EXPORTDLL TrAsyncSocket {
public:
  enum State {
    STATE_CLOSED = 0,      /*< Socket is not connected */
    STATE_OPEN             /*< Socket is connected */
  };

  /*******************************************************
   * socket_type -- TR_SOCK_STREAM or TR_SOCK_DGRAM
   *******************************************************/
  TrAsyncSocket(int socket_type);
  TrAsyncSocket(TrAsyncSocketHelper *helper);
  ~TrAsyncSocket();

  /*******************************************************
   * Get socket state
   *******************************************************/
  State state();

  /*******************************************************
   * Return winsock error code
   *******************************************************/
  int GetError();   

  void Bind(const TrSocketAddress* addr);  
  void Bind(unsigned short  port);
  void Listen(int backlog);

  void Connect(TrSocketAddress* addr);
  virtual void OnRead(char * data, int len);

  /*******************************************************
   * Happen as server mode
   * if receive a connect msg SDK call this function to
   * notify the App
   * helper -- for SDK, the App must pass this return to SDK
   * remote_addr -- remote endpoint address
   *******************************************************/
  virtual void OnAccept(TrAsyncSocketHelper *helper, TrSocketAddress remote_addr);

  void Write(const char * data, int len);
  void Close();

  virtual void OnSignalEvent(State s){}
  
  TrAsyncSocketHelper *GetHelper();
  friend class TrAsyncSocketHelper;

private:
	TrAsyncSocketHelper *helper_;   /*< for SDK */
	int socket_type_;               /*< SOCK_STREAM or SOCK_DGRAM */
};


/************************************************************************
*	Function :	Tr_Init
*
*	Parameters :
*		TrSocketAddress * stun_server ;	  - STUN server ip address
*		TrSocketAddress * relay_server ;  - Relay server ip address
*
*	Description :	Initialize this SDK, if App dont want to connect to remote
*       stun server or relay server NULL is acceptable
*
*	Return : int ;
*		T_EROOR - On Failure 
*		T_SUCCESS - On Success
*
*	Note : This function must be called before others APIs
************************************************************************/
EXPORTDLL int Tr_Init(TrSocketAddress *stun_server, TrSocketAddress *relay_server);

/************************************************************************
*	Function :	Tr_Destroy
*
*	Parameters :
*
*	Description : release SDK resources
*
*	Return : void
************************************************************************/
EXPORTDLL void Tr_Destroy();

/************************************************************************
*	Function :	Tr_CreateICESocket
*
*	Parameters : 
*       TrICESocket *socket: -- ICE session
*
*	Description : Create a ICE socket (session)
*
*	Return : int ;
*		T_EROOR - On Failure 
*		T_SUCCESS - On Success
************************************************************************/
EXPORTDLL int Tr_CreateICESocket(TrICESocket *socket);

/************************************************************************
*	Function :	Tr_DestroyICESocket
*
*	Parameters : 
*       TrICESocket *socket: -- ICE session
*
*	Description : Destory a ICE socket (session)
*
*	Return : void
************************************************************************/
EXPORTDLL void Tr_DestroyICESocket(TrICESocket *socket);

/************************************************************************
*	Function :	Tr_GetSDKInfo
*
*	Parameters : 
*
*	Description : Get SDK information
*
*	Return : info string
************************************************************************/
EXPORTDLL char* Tr_GetSDKInfo();

/************************************************************************
*	Function :	Tr_StartStunServer
*
*	Parameters : 
*       unsigned short  port: -- Listen on this port
*
*	Description : Start STUN server on this machine
*
*	Return : int ;
*		T_EROOR - On Failure 
*		T_SUCCESS - On Success
************************************************************************/
EXPORTDLL int Tr_StartStunServer(unsigned short  port);

/************************************************************************
*	Function :	Tr_StopStunServer
*
*	Parameters : 
*
*	Description : Stop STUN server on this machine
*
*	Return : int ;
*		T_EROOR - On Failure 
*		T_SUCCESS - On Success
************************************************************************/
EXPORTDLL int Tr_StopStunServer();

/************************************************************************
*	Function :	Tr_StartRelayServer
*
*	Parameters : 
*
*	Description : Start Relay server on this machine
*
*	Return : int ;
*		T_EROOR - On Failure 
*		T_SUCCESS - On Success
************************************************************************/
EXPORTDLL int Tr_StartRelayServer(unsigned short  port);

/************************************************************************
*	Function :	Tr_StartRelayServer
*
*	Parameters : 
*
*	Description : Stop Relay server on this machine
*
*	Return : int ;
*		T_EROOR - On Failure 
*		T_SUCCESS - On Success
************************************************************************/
EXPORTDLL int Tr_StopRelayServer();


#endif